package oracle;

import java.math.BigInteger;

import javassist.util.proxy.ProxyObject;

import javax.persistence.Cacheable;
import javax.persistence.Column;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.DiscriminatorType;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.Lob;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.Version;

import org.hibernate.annotations.BatchSize;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Parameter;
import org.hibernate.annotations.Proxy;

//import com.sun.codemodel.ClassType;

@Entity(name = "SubEntity")
@BatchSize(size = 100)
@Inheritance(strategy = InheritanceType.TABLE_PER_CLASS)
@DiscriminatorColumn(name = "CLASS_TYPE", discriminatorType = DiscriminatorType.STRING, length = 32)
@DiscriminatorValue("SubEntity")
@Proxy(lazy = true, proxyClass = SubEntity.class)
@Table(name = "SubEntity")
public class SubEntity extends SuperclassEntity
{
	@Transient
	private static final long serialVersionUID = 583308239L;

	@GenericGenerator(name = "MMSSeqIDGenerator", strategy = "sequence-identity", parameters = @Parameter(name = "sequence", value = "MMSSEQ"))
	@Id
	@GeneratedValue(generator = "MMSSeqIDGenerator")
	@Column(name = "SUBENTITYID", nullable = false, columnDefinition = "NUMBER(20)")
	private BigInteger key;

	@Column(name = "CODE", nullable = true, columnDefinition = "VARCHAR2(10)")
	private String code;

	@Column(name = "FLAGS", nullable = true, columnDefinition = "NUMBER(10)")
	private BigInteger flags;

	@Version
	@Column(name = "VERSION", nullable = false, columnDefinition = "NUMBER(5)")
	protected Long version;

	/*default*/ public SubEntity()
	{
	}
	
	public SubEntity(String clob, BigInteger flags)
	{
		setMyClob(clob);
		setUFlag(flags);
	}


	public BigInteger getKey()
	{
		return key;
	}


	/**
	 * Key Setter
	 * @param key neuer Schlssel
	 */
	protected void setKey(final BigInteger key)
	{
		this.key = key;
	}


	public void setCode(final String code)
	{
		if((this.code != null && this.code.equals(code)) || (this.code == null && code == null))
		{
			return;
		}
		this.code = code;
	}


	public String getCode()
	{
		return this.code;
	}


	public void setFlags(final BigInteger flags)
	{
		if(this.flags != null && this.flags.equals(flags))
		{
			return;
		}
		this.flags = flags;
	}


	public BigInteger getFlags()
	{
		if(this.flags != null)
		{
			return this.flags;
		}
		throw new IllegalStateException("Betriebsstelle: Not-Null-Feld Flags ist Null");
	}



	public void setVersion(final Long version)
	{
		if(this.version != null && this.version.equals(version))
		{
			return;
		}
		this.version = version;
	}


	public Long getVersion()
	{
		if(this.version != null)
		{
			return this.version;
		}
		throw new IllegalStateException("Betriebsstelle: Not-Null-Feld LetzteVersion ist Null");
	}

	public int hashCode()
	{
		if(this.getKey() != null)
		{
			return this.getKey().hashCode();
		}
		return System.identityHashCode(this);
	}


	public boolean equals(final Object other)
	{
		if(this == other)
		{
			return true;
		}
		if(other == null)
		{
			return false;
		}

		// (1) ClassType-Vergleich
		final Class<? extends Object> thisClass = (this instanceof ProxyObject) ? getClass().getSuperclass() : getClass();
		final Class<? extends Object> otherClass = (other instanceof ProxyObject) ? other.getClass().getSuperclass() : other.getClass();
		if(otherClass != thisClass)
		{
			return false;
		}

		// (2) Vergleich der Inhalte
		final SubEntity otherEntity = (SubEntity) other;
		if(this.getKey() != null) // (2a) Vergleich der Schlssel
		{
			return this.getKey().equals(otherEntity.getKey());
		}
//		else if(otherEntity.getKey() == null) // (2b) Vergleich der anderen Werte
//		{
//			return (EqualsBuilder.reflectionEquals(this, otherEntity));
//		}

		return false;
	}


	/**
	 * {@inheritDoc}
	 * @see java.lang.Object#toString()
	 */
	@Override
//	@NonNull
	public String toString()
	{
		if(getKey() != null)
		{
			return getKey().toString() + " version " + version.intValue();
		}
		else
		{
			return "not persisted " + getClass().getSimpleName() + " without ID";
		}
	}


}
